/*
 * ButtonBox - a program launcher
 *
 * Copyright (C) 2000 Paul Collins
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */

#ifndef BUTTON_H
#define BUTTON_H

#include <X11/Xlib.h>

#include "configuration.h"	/* For button_info */
#include "dispatcher.h"



typedef struct Button Button;

extern char* default_font;

typedef struct ButtonGlobals
{
	GC gc;
} ButtonGlobals;

struct Button 
{
	ButtonGlobals* globals;
	Dispatcher* dispatcher;
	Window window;
	char* title;
	char* command;
	Font font;
	int min_width;
	int min_height;
	unsigned long foreground_pixel;
	unsigned long background_pixel;
	unsigned long border_pixel;
};



Button* button_new (Dispatcher* dispatcher, Window parent, button_info* b);

void button_set_default_font (char* name);

Window button_get_window (Button*);

#endif /* BUTTON_H */
