/*
 * ButtonBox - a program launcher
 *
 * Copyright (C) 2000 Paul Collins
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 */

#ifndef CONFIGURATION_H
#define CONFIGURATION_H


struct configuration 
{
	int num_buttons;
	struct button_info* buttons;
};

struct button_info
{
	char* bi_text;
	char* bi_command;
	unsigned long bi_foreground_pixel;
	unsigned long bi_background_pixel;
	unsigned long bi_border_pixel;
	struct button_info* bi_next;
};

typedef struct configuration configuration;
typedef struct button_info button_info;

configuration* configuration_load_from_file (char* file);

char* configuration_get_value (char* name, char* class);

void configuration_free ();





#endif /* CONFIGURATION_H */
