#ifndef ASSERTIONS_H
#define ASSERTIONS_H

#ifdef WANT_DEBUG

#define I(expr) \
  do { if (!(expr)) abort (); } while (0)

#define N(expr) \
  do { if (expr) abort (); } while (0)

#define L(format, args...) \
  fprintf (stderr, format, ## args)

#else  /* WANT_DEBUG */

#define I(expr) \
  do { } while (0)

#define N(expr) \
  do { } while (0)

#define L(format, args...) \
  do { } while (0)

#endif /* not WANT_DEBUG */

#endif /* ASSERTIONS_H */
